/*	rsa_lib/rsa.h 	
	This RSA library is using BN module from OpenSSL Project (www.openssl.org)
	NOTE:
	1. This RSA library is stricted to only 2048 bits modulo value (=N).
	2. Only use two primes, P and Q (each 1024 bits), DOES NOT use multi prime.
	3. This RSA encryption library is DESIGNED for private individual or group use only.
	4. It follows the rules in PKCS#1 v2.1 (as per 5th January 2001).
	5. Use a slightly different OAEP algorithm, yet it possesses a strong cryptography.
	6. The strong/weak point is in BN_RAND_HL.C (random number generator based on time)
	7. No hash function.
	8. The certificate (public key) is NOT compatible for X509.
	Last compiled Tuesday, 13th November 2001 */

/* Written by Hariyanto Lim 
 * hmaxf_urlcr@yahoo.com
 * www.geocities.com/hmaxf_urlcr
 * ====================================================================
 * Copyright (c) 2001.  HMaxF Project, All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 3. Copyright remains to Hariyanto Lim, and as such any Copyright notices in
 *	  the code are not to be removed. If this package is used in a product, 
 *    Hariyanto Lim should be given attribution as the author of the parts of the library used.
 *    This can be in the form of a textual message at program startup or
 *    in documentation (online or textual) provided with the package.
 * ===================================================================
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND 
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
 * IN NO EVENT SHALL I, HARIYANTO LIM, THE AUTHOR BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 */

#ifndef RSA_HL_H
#define RSA_HL_H

#include <bn.h>
#include <bn_lcl.h>

#ifdef  __cplusplus
extern "C" {
#endif

// define max block size (= 2048 bits) ... because of the padding limitation (rule)
#define prime_bit_size					1024 // p and q each is 1024 bits ... n = 2048 bits
#define buffer_size						2048
#define max_block_size					256 // bytes
#define chiper_block_length_byte		256	// bytes
#define seed_length_bit					190 // for OAEP ... 190 bits = 24 bytes - 2 bits
#define seed_BN_ULONG_length			6 // 
#define message_length_bit				1856
#define chiper_BN_ULONG_length			64
#define message_BN_ULONG_length			58 // 58 blocks of BN_ULONG
#define BN_ULONG_length					4
#define message_block_length_byte		232 // message_BN_ULONG_length * BN_ULONG_length	

#define HMaxF_RSA_version_string		"HMaxF RSA 2048-bits v.1.1" // first version of the added value
#define HMaxF_RSA_version_string_length	25



typedef struct private_key
{	
	// e is only for re-generate public certificate ... in case public certificate is lost
	// e also use encryption using private key (= for private use)
	BIGNUM *p;
	BIGNUM *q;
	BIGNUM *dp;
	BIGNUM *dq;
	BIGNUM *e;
	BIGNUM *qinv;
} pri_key;

typedef struct public_key
{
	BIGNUM *e;
	BIGNUM *n;
} pub_key;

// rsa_encode.c
long RSA_public_certificate_do_1_block(BIGNUM *target, BIGNUM *msg, pub_key *key, BN_CTX *ctx);
long RSA_private_key_CRT_do_1_block(BIGNUM *target, BIGNUM *msg, pri_key *key, BN_CTX *ctx);
unsigned long RSA_remove_padding(unsigned char *source, unsigned long source_size);
unsigned long RSA_insert_padding(unsigned char *source, unsigned long source_size);
long OAEP_decode(BIGNUM *source, BIGNUM *target);
long OAEP_encode(BIGNUM *source, BIGNUM *target);
unsigned long __stdcall RSA_encrypt(unsigned char *source, unsigned long source_size, 
								    unsigned char *result, unsigned long result_size, 
								    unsigned char *pubkey_content, unsigned long pubkey_content_size);
unsigned long __stdcall RSA_decrypt(unsigned char *source, unsigned long source_size,
									unsigned char *result, unsigned long result_size, 
									unsigned char *prikey_content, unsigned long prikey_content_size);
unsigned long __stdcall RSA_sign(unsigned char *source, unsigned long source_size, 
								 unsigned char *result, unsigned long result_size, 
								 unsigned char *prikey_content, unsigned long prikey_content_size);
unsigned long __stdcall RSA_verify(unsigned char *source, unsigned long source_size, 
								   unsigned char *result, unsigned long result_size, 
								   unsigned char *pubkey_content, unsigned long pubkey_content_size);
						   
// rsa_keys.c
unsigned long __stdcall RSA_generate_keys(unsigned char *privatekey, unsigned char *publickey, 
										  unsigned long *privatekey_length, unsigned long *publickey_length);
unsigned long convert_private_key_to_PEM_char(pri_key *prikey, unsigned char *target);
unsigned long convert_public_certificate_to_PEM_char(pub_key *pubkey, unsigned char *target);
pri_key *RSA_read_private_key(unsigned char *source, unsigned long source_length);
pub_key *RSA_read_public_certificate(unsigned char *source, unsigned long source_length);

// rsa_lib.c
unsigned long __stdcall HMaxF_RSA_version(unsigned char *target, unsigned long target_size);
pri_key *RSA_private_key_new();
void RSA_private_key_free(pri_key **prikey);
pub_key *RSA_public_certificate_new();
void RSA_public_certificate_free(pub_key **pubkey);

// rsa_pem.c
unsigned long PEM_encode(unsigned char *source, long source_size, unsigned char *target);
unsigned long PEM_decode(unsigned char *source, long source_size, unsigned char *target);

#ifdef  __cplusplus
}
#endif
#endif
