/*	rsa_lib/rsa_keys.c
	This RSA library is using BN module from OpenSSL Project (www.openssl.org)
	NOTE:	
	1. This RSA library is stricted to only 2048 bits modulo value (=N).
	2. Only use two primes, P and Q (each 1024 bits), DOES NOT use multi prime.
	3. This RSA encryption library is DESIGNED for private individual or group use only.
	4. It follows the rules in PKCS#1 v2.1 (as per 5th January 2001).
	5. Use a slightly different OAEP algorithm, yet it possesses a strong cryptography.
	6. The strong/weak point is in BN_RAND_HL.C (random number generator based on time)
	7. No hash function.
	8. The certificate (public key) is NOT compatible for X509.
	Last compiled Tuesday, 13th November 2001 */

/* Written by Hariyanto Lim 
 * hmaxf_urlcr@yahoo.com
 * www.geocities.com/hmaxf_urlcr
 * ====================================================================
 * Copyright (c) 2001.  HMaxF Project, All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 3. Copyright remains to Hariyanto Lim, and as such any Copyright notices in
 *	  the code are not to be removed. If this package is used in a product, 
 *    Hariyanto Lim should be given attribution as the author of the parts of the library used.
 *    This can be in the form of a textual message at program startup or
 *    in documentation (online or textual) provided with the package.
 * ===================================================================
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND 
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
 * IN NO EVENT SHALL I, HARIYANTO LIM, THE AUTHOR BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 */

#include <stdio.h>
#include <bn.h>
#include <rsa.h>
#include <malloc.h> // malloc()
#include <memory.h> // memcpy()
#include <string.h> // strncmp()

#define buffer_size					2048
#define bn_header_size				2      
#define certificate_top				"-----BEGIN CERTIFICATE-----"
#define certificate_bottom			"-----END CERTIFICATE-----"
#define key_top						"-----BEGIN RSA PRIVATE KEY-----"
#define key_bottom					"-----END RSA PRIVATE KEY-----"
#define certificate_top_length		27
#define certificate_bottom_length	25
#define key_top_length				31
#define key_bottom_length			29

// use constant because memcpy() will give warning if use #define
const unsigned long crlf_length =	2;
const unsigned char crlf[] =		{ 0x0D, 0x0A};

// this is function is simply what it said
// return 0 (zero) if success, *privatekey and *publiccertificate MUST be preallocated and enough size
unsigned long __stdcall RSA_generate_keys(unsigned char *privatekey, unsigned char *publiccertificate, 
										  unsigned long *privatekey_length, unsigned long *publiccertificate_length)
{
	pri_key *prikey = NULL;
	pub_key *pubkey = NULL;
	BIGNUM *p1 = BN_new();
	BIGNUM *q1 = BN_new();
	BIGNUM *temp = BN_new();	
	BN_CTX *ctx = BN_CTX_new();

	prikey = RSA_private_key_new();
	if(prikey == NULL)
		goto err;

	pubkey = RSA_public_certificate_new();
	if(pubkey == NULL)
		goto err;
	
	// generate a fix e = 65537
	if(!BN_set_word(prikey->e, 65537))
		goto err;

generate_key:	

	// generate p	
	prikey->p = BN_generate_prime(NULL, prime_bit_size, 0, NULL, NULL, NULL, NULL);
	if(prikey->p == NULL)
		goto err;

	// generate q	
	prikey->q = BN_generate_prime(NULL, prime_bit_size, 0, NULL, NULL, NULL, NULL);	
	if(prikey->q == NULL)
		goto err;

	if(!BN_cmp(prikey->p, prikey->q))
		goto generate_key; // p = q ... this is very rare happen .. the chance is very small
	
	// the different between p and q must be large .. ignore the negative flag
	BN_sub(temp, prikey->p, prikey->q);
	if(temp->top < 25)
		goto generate_key;

	// generate p1 = p - 1
	BN_sub(p1, prikey->p, BN_value_one());

	// generate q1 = q - 1
	BN_sub(q1, prikey->q, BN_value_one());

	// check whether p and q is valid for this fix e
	BN_gcd(temp, prikey->e, p1, ctx);
	if(!BN_is_word(temp, 1))
		goto generate_key;

	BN_gcd(prikey->qinv, prikey->e, q1, ctx); // use qinv .. prevent double use
	if(!BN_is_word(prikey->qinv, 1))
		goto generate_key;

	// generate dp	
	if(BN_mod_inverse(prikey->dp, prikey->e, p1, ctx) == NULL)
		goto err;

	// generate dq	
	if(BN_mod_inverse(prikey->dq, prikey->e, q1, ctx) == NULL)
		goto err;

	// generate qinv	
	if(BN_mod_inverse(prikey->qinv, prikey->q, prikey->p, ctx) == NULL)
		goto err;

	// store all the variable required in public certificate structure	=======================
	if(BN_copy(pubkey->e, prikey->e) == NULL)
		goto err;
	if(!BN_mul(pubkey->n, prikey->p, prikey->q, ctx))
		goto err;
		
	// convert rsa private key structure to char*
	*privatekey_length = convert_private_key_to_PEM_char(prikey, privatekey);
	if(*privatekey_length == 0)
		goto err;

	// convert rsa public certificate structure to char*
	*publiccertificate_length = convert_public_certificate_to_PEM_char(pubkey, publiccertificate);
	if(*publiccertificate_length == 0)
		goto err;

	// all allocated variables must be free here before exit
	RSA_public_certificate_free(&pubkey);
	RSA_private_key_free(&prikey);
	BN_free(p1);
	BN_free(q1);	
	BN_free(temp);
	BN_CTX_free(ctx);

	return(0); 
err:
	RSA_public_certificate_free(&pubkey);
	RSA_private_key_free(&prikey);
	BN_free(p1);
	BN_free(q1);	
	BN_free(temp);
	BN_CTX_free(ctx);

	return((unsigned long)-1); // error
}

// WARNING: assign enough space for target, no check for overrun ..
// return the target REAL size, if zero then error
// this function use array to avoid the use of malloc which is unsafe (and buggy)
// especially in mixed language like C/C++ and Visual Basic
unsigned long convert_private_key_to_PEM_char(pri_key *prikey, unsigned char *target)
{
	unsigned int result = 0;
	unsigned int total = 0;
	unsigned char PEM_buffer[buffer_size];// = NULL;
	unsigned long target_size = 0;
	unsigned long PEM_buffer_length = 0;
	unsigned char *buffer = target;	// use direct 'target' as temp buffer
	
	//save dp	
	result = BN_bn2bin(prikey->dp, buffer + bn_header_size);
	memcpy(buffer, &result, bn_header_size);	// store header	
	buffer += (bn_header_size + result);		// increase buffer pointer	
	total += (result + bn_header_size);

	//save dq
	result = BN_bn2bin(prikey->dq, buffer + bn_header_size);
	memcpy(buffer, &result, bn_header_size);	// store header	
	buffer += (bn_header_size + result);		// increase buffer pointer	
	total += (result + bn_header_size);

	//save e
	result = BN_bn2bin(prikey->e, buffer + bn_header_size);
	memcpy(buffer, &result, bn_header_size);	// store header	
	buffer += (bn_header_size + result);		// increase buffer pointer	
	total += (result + bn_header_size);

	//save p
	result = BN_bn2bin(prikey->p, buffer + bn_header_size);
	memcpy(buffer, &result, bn_header_size);	// store header	
	buffer += (bn_header_size + result);		// increase buffer pointer	
	total += (result + bn_header_size);

	//save q
	result = BN_bn2bin(prikey->q, buffer + bn_header_size);
	memcpy(buffer, &result, bn_header_size);	// store header	
	buffer += (bn_header_size + result);		// increase buffer pointer	
	total += (result + bn_header_size);

	//save qinv
	result = BN_bn2bin(prikey->qinv, buffer + bn_header_size);
	memcpy(buffer, &result, bn_header_size);	// store header	
	buffer += (bn_header_size + result);		// increase buffer pointer	
	total += (result + bn_header_size);

	PEM_buffer_length = PEM_encode(target, total, PEM_buffer);
	if(PEM_buffer_length == 0)
		return(0);

	// store the top header .. use the 'target' variable direct
	memcpy(target, key_top, key_top_length);			// store header	key_top
	memcpy(target + key_top_length, crlf, crlf_length);	// add a crlf
	
	// store the private key
	memcpy(target + key_top_length + crlf_length, PEM_buffer, PEM_buffer_length);

	// store the bottom header tag
	memcpy(target + key_top_length + crlf_length + PEM_buffer_length, crlf, crlf_length);// add a crlf	
	memcpy(target + key_top_length + crlf_length + PEM_buffer_length + crlf_length, key_bottom, key_bottom_length);
		
	target_size = (PEM_buffer_length + key_top_length + key_bottom_length + (2 * crlf_length));

	return(target_size);
}

unsigned long convert_public_certificate_to_PEM_char(pub_key *pubkey, unsigned char *target)
{
	unsigned int result = 0;
	unsigned int total = 0;
	unsigned char PEM_buffer[buffer_size];
	unsigned long PEM_buffer_length = 0, target_size = 0;
	unsigned char *buffer = target;

	//save e
	result = BN_bn2bin(pubkey->e, buffer + bn_header_size);
	memcpy(buffer, &result, bn_header_size);	// store header	
	buffer += (bn_header_size + result);		// increase buffer pointer	
	total += (result + bn_header_size);

	//save n
	result = BN_bn2bin(pubkey->n, buffer + bn_header_size);
	memcpy(buffer, &result, bn_header_size);	// store header	
	buffer += (bn_header_size + result);		// increase buffer pointer	
	total += (result + bn_header_size);

	//buffer = original_buffer;	// rewind

	PEM_buffer_length = PEM_encode(target, total, PEM_buffer);
	if(PEM_buffer_length == 0)
		return(0);

	// store the top header ... re-use buffer variable
	memcpy(target, certificate_top, certificate_top_length);		// store header	key_top
	memcpy(target + certificate_top_length, crlf, crlf_length);	// add a crlf
	
	// store the private key
	memcpy(target + certificate_top_length + crlf_length, PEM_buffer, PEM_buffer_length);

	// store the bottom header tag
	memcpy(target + certificate_top_length + crlf_length + PEM_buffer_length, crlf, crlf_length);// add a crlf	
	memcpy(target + certificate_top_length + crlf_length + PEM_buffer_length + crlf_length, certificate_bottom, certificate_bottom_length);
		
	target_size = (PEM_buffer_length + certificate_top_length + certificate_bottom_length + (2 * crlf_length));

	return(target_size);
}

pri_key *RSA_read_private_key(unsigned char *source, unsigned long source_length)
{
	//unsigned long result = 0;	
	unsigned char certificate[buffer_size];
	unsigned long cert_size;
	unsigned long bnsize;
	unsigned long totalread;
	pri_key *prikey = NULL; // must be unallocated otherwise memory leak

	// allocate structure storage
	prikey = RSA_private_key_new();
	if(prikey == NULL)
		return(NULL);

	totalread = 0; // reset
	bnsize = 0;

	// take out the top header
	if(strncmp(source, key_top, key_top_length) != 0)
		goto err;	

	// take out the bottom header
	if(strncmp(source + (source_length - key_bottom_length), key_bottom, key_bottom_length) != 0)
		goto err;	

	// translate to real binary value of certificate
	cert_size = PEM_decode(source + (key_top_length + crlf_length), source_length - (key_top_length + key_bottom_length + (2 * crlf_length)), certificate);
	if(cert_size == 0)
		goto err;

	//get size and read dp
	memcpy(&bnsize, certificate + totalread, bn_header_size);		
	totalread += bn_header_size; // update buffer read pointer		
	if(BN_bin2bn(certificate + totalread, bnsize, prikey->dp) == NULL)
		goto err;
	totalread += bnsize;
	
	//get size and read dq
	memcpy(&bnsize, certificate + totalread, bn_header_size);		
	totalread += bn_header_size; // update buffer read pointer		
	if(BN_bin2bn(certificate + totalread, bnsize, prikey->dq) == NULL)
		goto err;
	totalread += bnsize;

	//get size and read e
	memcpy(&bnsize, certificate + totalread, bn_header_size);	
	totalread += bn_header_size; // update buffer read pointer
	if(BN_bin2bn(certificate + totalread, bnsize, prikey->e) == NULL)
		goto err;
	totalread += bnsize;

	//get size and read p
	memcpy(&bnsize, certificate + totalread, bn_header_size);	
	totalread += bn_header_size; // update buffer read pointer
	if(BN_bin2bn(certificate + totalread, bnsize, prikey->p) == NULL)
		goto err;
	totalread += bnsize;

	//get size and read q
	memcpy(&bnsize, certificate + totalread, bn_header_size);	
	totalread += bn_header_size; // update buffer read pointer
	if(BN_bin2bn(certificate + totalread, bnsize, prikey->q) == NULL)
		goto err;
	totalread += bnsize;

	//get size and read qinv
	memcpy(&bnsize, certificate + totalread, bn_header_size);	
	totalread += bn_header_size; // update buffer read pointer
	if(BN_bin2bn(certificate + totalread, bnsize, prikey->qinv) == NULL)
		goto err;
	totalread += bnsize;

	// finished ... comparing the totalread with actual file size
	if(totalread != cert_size)
		goto err;

	return(prikey);

err:
	RSA_private_key_free(&prikey);

	return(NULL);
}

pub_key *RSA_read_public_certificate(unsigned char *source, unsigned long source_length)
{
	//unsigned long result = 0;	
	unsigned char certificate[buffer_size];
	unsigned long cert_size;
	unsigned long bnsize;
	unsigned long totalread;
	pub_key *pubkey = NULL;

	// allocate structure storage
	pubkey = RSA_public_certificate_new();
	if(pubkey == NULL)
		return(NULL);

	// check the top header
	if(strncmp(source, certificate_top, certificate_top_length) != 0)
		goto err;

	// check the bottom header
	if(strncmp(source + (source_length - certificate_bottom_length), certificate_bottom, certificate_bottom_length) != 0)
		goto err;

	// translate to real binary value of certificate
	cert_size = PEM_decode(source + (certificate_top_length + crlf_length), source_length - (certificate_top_length + certificate_bottom_length + (2 * crlf_length)), certificate);
	if(cert_size == 0)
		goto err;

	totalread = 0; // reset
	bnsize =0;

	//get size and read e
	memcpy(&bnsize, certificate + totalread, bn_header_size);
	totalread += bn_header_size; // update buffer read pointer
	if(BN_bin2bn(certificate + totalread, bnsize, pubkey->e) == NULL)
		goto err;
	totalread += bnsize;

	//get size and read n
	memcpy(&bnsize, certificate + totalread, bn_header_size);
	totalread += bn_header_size; // update buffer read pointer
	if(BN_bin2bn(certificate + totalread, bnsize, pubkey->n) == NULL)
		goto err;
	totalread += bnsize;	

	// finished ... comparing the totalread with actual file size
	if(totalread != cert_size)
		goto err;

	return(pubkey);

err:
	RSA_public_certificate_free(&pubkey);

	return(NULL);
}